var chai = require('chai'),
  assert = chai.assert,
  heroin = require('../lib/heroin'),
  _ = require('lodash');

describe('HeroIn pipelines', function () {

  it('should prompt you for a pipeline config or name', function () {
    try {
      var configurator = heroin({});
      configurator.pipeline();
    } catch (e) {
      assert.equal(e.message, 'Please specify pipeline name or pipeline config');
    }
  });

  it('should not allow to create pipeline without any apps - no apps', function () {
    try {
      var configurator = heroin({});
      configurator.pipeline({name: 'sample_pipeline'});
    } catch (e) {
      assert.equal(e.message, 'Please specify at least one app in your pipeline. Provided: undefined');
    }
  });

  it('should not allow to create pipeline without any apps - empty apps', function () {
    try {
      var configurator = heroin({});
      configurator.pipeline({name: 'sample_pipeline', apps: {}});
    } catch (e) {
      assert.equal(e.message, 'Please specify at least one app in your pipeline. Provided: {}');
    }
  });

  it('should allow to export existing pipeline', function (done) {
    var configurator = heroin(inMemoryHerokuClient());
    configurator.
      pipeline('sample_pipeline').
      then(function (pipelineConfig) {
        assert.deepEqual({name: 'sample_pipeline', apps: {production: 'sample_app2'}}, pipelineConfig);
      }).
      then(done).
      catch(done);
  });

  it('should allow to create/update pipelines', function (done) {
    var configurator = heroin(inMemoryHerokuClient(), {logLevel: 'ERROR'});
    var pipelineConfig = {
      name: 'sample_pipeline',
      apps: {staging: 'sample_app1', production: 'sample_app2'}
    };
    var updatedPipelineConfig = {
      name: 'sample_pipeline',
      apps: {review: 'sample_app2', staging: 'sample_app1'}
    };
    configurator.
      pipeline(pipelineConfig).
      then(function () {
        return configurator.pipeline('sample_pipeline');
      }).
      then(function (actualPipelineConfig) {
        assert.deepEqual(actualPipelineConfig, pipelineConfig);
      }).
      then(function() {
        return configurator.pipeline(updatedPipelineConfig);
      }).
      then(function() {
        return configurator.pipeline('sample_pipeline');
      }).
      then(function (actualPipelineConfig) {
        assert.deepEqual(actualPipelineConfig, updatedPipelineConfig);
      }).
      then(done).
      catch(done);
  });

  it('should allow to put multiple apps in one pipeline', function (done) {
    var configurator = heroin(inMemoryHerokuClient(), {logLevel: 'ERROR'});
    var pipelineConfig = {
      name: 'sample_pipeline',
      apps: {production: ['sample_app1', 'sample_app2']}
    };
    configurator.
      pipeline(pipelineConfig).
      then(function () {
        return configurator.pipeline('sample_pipeline');
      }).
      then(function (actualPipelineConfig) {
        assert.deepEqual(actualPipelineConfig, pipelineConfig);
      }).
      then(done).
      catch(done);
  });

  it('should be able to add apps to pipeline', function (done) {
    var configurator = heroin(inMemoryHerokuClient(), {logLevel: 'ERROR'});
    var pipelineConfig = {
      name: 'sample_pipeline',
      apps: {production: 'sample_app1'}
    };

    configurator.pipeline(pipelineConfig).
      then(function () {
        return configurator.addToPipeline({
          name: 'sample_pipeline',
          apps: {production: 'sample_app2'}
        });
      }).
      then(function () {
        return configurator.pipeline('sample_pipeline');
      }).
      then(function (actualPipelineConfig) {
        assert.deepEqual(actualPipelineConfig, {
          name: 'sample_pipeline',
          apps: {production: ['sample_app1', 'sample_app2']}
        });
      }).
      then(done).
      catch(done);
  });

  it('should treat adding operation as creation if pipeline does not exist', function (done) {
    var configurator = heroin(inMemoryHerokuClient(), {logLevel: 'ERROR'});
    var pipelineConfig = {
      name: 'sample_pipeline2',
      apps: {staging: 'sample_app1'}
    };
    configurator.
      addToPipeline(pipelineConfig).
      then(function () {
        return configurator.pipeline('sample_pipeline2');
      }).
      then(function (actualPipelineConfig) {
        assert.deepEqual(actualPipelineConfig, pipelineConfig);
      }).
      then(done).
      catch(done);
  });

  function inMemoryHerokuClient() {
    var herokuClient = {
      pipelinesList: {sample_pipeline: {id: 'sample_pipeline', name: 'sample_pipeline'}},
      appsList: {sample_app1: {name: 'sample_app1'}, sample_app2: {name: 'sample_app2'}},
      couplings: {
        'sample_pipeline': [{
          app: {
            id: 'sample_app2',
          },
          pipeline: {
            id: 'sample_pipeline'
          },
          stage: 'production'
        }]
      },

      apps: function (appId) {
        return {
          info: function () {
            return Promise.resolve(herokuClient.appsList[appId]);
          }
        };
      },

      pipelineCouplings: function (couplingId) {
        return {
          create: function (coupling) {
            if (!herokuClient.couplings[coupling.pipeline]) {
              herokuClient.couplings[coupling.pipeline] = [];
            }
            herokuClient.couplings[coupling.pipeline].push({
              id: Math.ceil(Math.random() * 10000),
              app: {
                id: coupling.app
              },
              stage: coupling.stage,
              pipeline: {
                id: coupling.pipeline
              }
            });
            return Promise.resolve();
          },
          delete: function () {
            Object.keys(herokuClient.couplings).forEach(function (couplingsGroup) {
              herokuClient.couplings[couplingsGroup] = herokuClient.couplings[couplingsGroup].
                filter(function (coupling) {
                  return coupling.id !== couplingId;
                });
            });
          }
        };
      },

      pipelines: function (name) {
        return {
          pipelineCouplings: function () {
            return {
              list: function () {
                return Promise.resolve(herokuClient.couplings[name]);
              }
            };
          },
          delete: function () {
            delete herokuClient.pipelinesList[name];
            herokuClient.couplings[name] = [];
            return Promise.resolve();
          },
          create: function (pipelineConfig) {
            var pipeline = {id: pipelineConfig.name, name: pipelineConfig.name};
            herokuClient.pipelinesList[pipelineConfig.name] = pipeline;
            return Promise.resolve(pipeline);
          },
          info: function () {
            if (herokuClient.pipelinesList[name]) {
              return Promise.resolve(herokuClient.pipelinesList[name]);
            }
            return Promise.reject();
          }
        };
      }
    };
    return herokuClient;
  }
});
